# pylint: disable=missing-function-docstring

"""
Usage:
    anaconda download notebook
    anaconda download user/notebook
"""

from __future__ import unicode_literals

import argparse
import logging

from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.config import PackageType
from binstar_client.utils.notebook import parse, has_environment
from binstar_client.utils.notebook.downloader import Downloader

logger = logging.getLogger('binstar.download')


def add_parser(subparsers):
    description = 'Download notebooks from your Anaconda repository'
    parser = subparsers.add_parser(
        'download',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        help=description,
        description=description,
        epilog=__doc__
    )

    parser.add_argument(
        'handle',
        help='user/notebook',
        action='store'
    )

    parser.add_argument(
        '-f', '--force',
        help='Overwrite',
        action='store_true'
    )

    parser.add_argument(
        '-o', '--output',
        help='Download as',
        default='.'
    )
    pkg_types = ', '.join(pkg_type.value for pkg_type in PackageType)
    parser.add_argument(
        '-t', '--package-type',
        help='Set the package type [{0}]. Defaults to downloading all package types available'.format(pkg_types),
        action='append',
    )
    parser.set_defaults(main=main)


def main(args):
    aserver_api = get_server_api(args.token, args.site)
    username, notebook = parse(args.handle)
    username = username or aserver_api.user()['login']
    downloader = Downloader(aserver_api, username, notebook)
    packages_types = list(map(PackageType, args.package_type) if args.package_type else PackageType)

    try:
        download_files = downloader.list_download_files(packages_types, output=args.output, force=args.force)
        for download_file, download_dist in download_files.items():
            downloader.download(download_dist)
            logger.info('%s has been downloaded as %s', args.handle, download_file)
            if has_environment(download_file):
                logger.info('%s has an environment embedded.', download_file)
                logger.info('Run:')
                logger.info('    conda env create %s', download_file)
                logger.info('To install the environment in your system')
    except (errors.DestinationPathExists, errors.NotFound, errors.BinstarError, OSError) as err:
        logger.info(err)



# pylint: disable=missing-module-docstring,missing-class-docstring,missing-function-docstring

import logging

from ..yaml import yaml_load

logger = logging.getLogger('binstar.auth')


class ProjectFilesInspector:  # pylint: disable=too-few-public-methods
    def __init__(self, pfiles):
        self.pfiles = pfiles

    def update(self, metadata):
        metadata['files'] = [pfile.to_dict() for pfile in self.pfiles]
        return metadata


class DocumentationInspector:
    valid_names = [
        'README.md',
        'README.rst',
        'README.txt',
        'README'
    ]

    def __init__(self, pfiles):
        self.pfiles = pfiles
        self.doc_pfile = None

    def update(self, metadata):
        if self.has_doc():
            with open(self.doc_pfile.fullpath) as docfile:  # pylint: disable=unspecified-encoding
                metadata['readme'] = docfile.read()
        return metadata

    def has_doc(self):
        def is_readme(basename, relativepath, fullpath):  # pylint: disable=unused-argument
            return basename == relativepath and basename in self.valid_names

        for pfile in self.pfiles:
            if pfile.validate(is_readme):
                self.doc_pfile = pfile
                break

        return self.doc_pfile is not None


class ConfigurationInspector:
    valid_names = [
        'project.yml',
        'project.yaml'
    ]

    def __init__(self, pfiles):
        self.pfiles = pfiles
        self.config_pfile = None

    def update(self, metadata):
        try:
            if self.has_config():
                with open(self.config_pfile.fullpath) as configfile:  # pylint: disable=unspecified-encoding
                    metadata['configuration'] = yaml_load(configfile)
        except Exception:  # pylint: disable=broad-except
            logger.warning('Could not parse configuration file.')
        return metadata

    def has_config(self):
        def is_config(basename, relativepath, fullpath):  # pylint: disable=unused-argument
            return basename == relativepath and basename in self.valid_names

        for pfile in self.pfiles:
            if pfile.validate(is_config):
                self.config_pfile = pfile
                break

        return self.config_pfile is not None


inspectors = [
    DocumentationInspector,
    ProjectFilesInspector,
    ConfigurationInspector
]



# pylint: disable=missing-function-docstring

"""
Print the information of the current user
"""

from __future__ import unicode_literals

import logging

from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.pprint import pprint_user

logger = logging.getLogger('binstar.whoami')


def main(args):  # pylint: disable=inconsistent-return-statements
    aserver_api = get_server_api(args.token, args.site)

    try:
        user = aserver_api.user()
    except errors.Unauthorized as err:
        logger.debug(err)
        logger.info('Anonymous User')
        return 1

    pprint_user(user)


def add_parser(subparsers):
    subparser = subparsers.add_parser('whoami',
                                      help='Print the information of the current user',
                                      description=__doc__)

    subparser.set_defaults(main=main)
